local S = silly_cheese.S

silly_cheese.on = {}
silly_cheese.on.eat = function(amount)
	return function (itemstack, user, pointed_thing)
		return minetest.do_item_eat(amount, silly_cheese.bucket.empty, itemstack, user, pointed_thing)
	end
end

silly_cheese.register = {}

silly_cheese.register.milk = function(name, description, texture, heals)
	if minetest.registered_craftitems[silly_cheese.origmod..":"..name] ~= nil then return end

	minetest.register_craftitem(
		":"..silly_cheese.origmod..":"..name,
		{
			description = S(description),
			groups = silly_cheese.bucket.groups,
			stack_max = 99,

			wield_image = texture,    
			inventory_image = texture,

			on_use = silly_cheese.on.eat(heals),
		}
	)
	return silly_cheese.origmod..":"..name
end

silly_cheese.register.milk_boiling = function(milk, curd, whey, ...)
  if cheese.ui then
      unified_inventory.register_craft({
        type = "cauldron_boiling",
        items = {milk},
        output = curd
      })
      if v[3] ~= nil then
        unified_inventory.register_craft({
    			type = "cauldron_boiling",
    			items = {milk},
    			output = whey
    		})
      end
  end

  if cheese.i3 then
    i3.register_craft({
      type = "cauldron_boiling",
      items = {milk},
      result = v[2]
    })
    if v[3] ~= nil then
      i3.register_craft({
  			type = "cauldron_boiling",
  			items = {milk},
  			result = whey
  		})
    end
  end

end

silly_cheese.register.curd = function(name, description, texture)
	if minetest.registered_craftitems[silly_cheese.origmod..":"..name] ~= nil then return end

	minetest.register_craftitem(
		":"..silly_cheese.origmod..":"..name,
		{
			description = S(description),
			inventory_image = texture,
			groups = {milk_product = 1},
		}
	)
	return silly_cheese.origmod..":"..name
end

silly_cheese.register.cheese = function(name, description, texture, curd)
	if minetest.registered_craftitems[silly_cheese.origmod..":"..name] ~= nil then return end

	minetest.register_craftitem(
		":"..silly_cheese.origmod..":"..name,
		{
			description = S(description),
			inventory_image = texture,
			on_use = minetest.item_eat(2),
			groups = {food = 2, food_cheese = 1},
		}
	)
	if cheese.ui then
		unified_inventory.register_craft({
			type = "cheese_rack_aging",
			items = {curd},
			output = silly_cheese.origmod..":"..name
		})
	end
	if cheese.i3 then
		i3.register_craft({
			type = "cheese_rack_aging",
			items = {curd},
			result = silly_cheese.origmod..":"..name
		})
	end
	if cheese.hunger_ng then
		hunger_ng.add_hunger_data(silly_cheese.origmod..":"..name , { satiates = 2, heals = 0, timeout = 0 })
	end

	return silly_cheese.origmod..":"..name
end

local function get_rack(material, suffix)
	return "cheese:"..material.."_cheese_rack_"..suffix
end

silly_cheese.register.aging = function(curd, cheese)
	for _, rack in ipairs({"stone", "wooden", "mossy"}) do
		for __, suffix in ipairs({"empty", "with_aging_cheese", "with_cheese"}) do
			local rack_name = get_rack(rack, suffix)
			local the_rack = minetest.registered_nodes[rack_name]
			if the_rack == nil then
				goto continue
			end

			--------------------------------------------------------------------------
			-- On right click
			--------------------------------------------------------------------------

			local onrightclick = the_rack.on_rightclick

			local the_rack_on_rightclick_wrapper = function(pos, node, player, itemstack, pointed_thing)
				if player:is_player() and itemstack:get_name() == curd then
					itemstack:take_item()
					minetest.sound_play(
						"ftsp",
						{pos = pos, max_hear_distance = 8, gain = 0.5}
					)
	
					minetest.set_node(pos, {name = get_rack(rack, "with_aging_cheese"), param2 = node.param2})
				
					local meta = minetest.get_meta(pos)
					meta:set_string("aging", itemstack:get_name())
					minetest.chat_send_player("singleplayer", "Setting meta aging to "..itemstack:get_name())
				else
					onrightclick(pos, node, player, itemstack, pointed_thing)
				end
				return itemstack
			end

			the_rack.on_rightclick = the_rack_on_rightclick_wrapper

			--------------------------------------------------------------------------
			-- On punch
			--------------------------------------------------------------------------

			local onpunch = the_rack.on_punch

			local on_punch_wrapper = function(pos, node, player, pointed_thing)
				if not player:is_player() then return end

				local given = cheese.." 4"

				local meta = minetest.get_meta(pos)
				minetest.chat_send_player("singleplayer", "Found meta aging "..meta:get_string("aging"))

				if meta:get_string("aging") == curd then
					local inv = player:get_inventory()

					minetest.sound_play("ftspw", {pos = pos, max_hear_distance = 8, gain = 0.5})

					if inv:room_for_item("main", given) then
						local leftover = inv:add_item("main", given)
						if not leftover:is_empty() then
							minetest.add_item(player:get_pos(), leftover)
						end
					else
						minetest.add_item(player:get_pos(), given)
					end
					minetest.set_node(pos, {name = get_rack(rack, "empty"), param2 = node.param2})
				else
					onpunch(pos, node, player, pointed_thing)
				end
			end

			the_rack.on_punch = on_punch_wrapper

			-- no more code under this line!

			::continue::

		end  -- end for suffix
	end  -- end for material
end


silly_cheese.patch_petz = function(name, feed, yields, ...)
	local pet = minetest.registered_entities[name]
	if pet == nil then return end

	pet.milkable = true
	-- pet.is_male = false  -- For debug only!
	pet.follow = pet.follow or feed

	local pet_orig_onrightclick = pet.on_rightclick

	local function pet_on_rightclick_wrapper(self, clicker)
		 local inv = clicker:get_inventory()
		 local wielded_item = clicker:get_wielded_item()
		 local wielded_item_name = wielded_item:get_name()

		 if wielded_item_name == silly_cheese.bucket.empty and clicker:get_inventory() then
		      if not(self.milked) then
				 wielded_item:take_item()
				 clicker:set_wielded_item(wielded_item)
				 if inv:room_for_item("main", yields) then
					  inv:add_item("main", yields)
					  kitz.make_sound(
						  "object",
						  self.object,
						  "petz_"..self.type.."_moaning",
						  petz.settings.max_hear_distance
					  )
				 else
					  minetest.add_item(
						  self.object:get_pos(),
						  yields
					  )
				 end
				 self.milked = kitz.remember(self, "milked", true)
		      else
		           minetest.chat_send_player(
				       clicker:get_player_name(),
				       "This animal has already been milked."
		           )
		      end
		 else
		      pet_orig_onrightclick(self, clicker)
		 end
	end

	pet.on_rightclick = beaver_on_rightlick_wrapper
end

silly_cheese._internal = {}
-- https://stackoverflow.com/questions/22831701/lua-read-beginning-of-a-string
silly_cheese._internal.startswith = function(String, Start)
   return string.sub(String, 1, string.len(Start)) == Start
end

